'
'*************************************************************************************************************
'* This source code is provided 'as is' and any express or implied warranties, including,                    *
'* but not limited to, the implied warranties of merchantability and fitness for a particular                *
'* purpose are disclaimed.                                                                                   *
'*                                                                                                           *
'* In no event shall the author, Crownhill or contributers be liable for any direct, indirect, incidental,   *
'* special, exemplary, or consequential damages (including, but not limited to,  procurement of substitute   *
'* goods or services; loss of use, data, or profits;  or business interruption) however caused and on any    *
'* theory of liability, whether in contract, strict liability, or tort (including negligence or otherwise)   *
'* arising in any way out of the use of this source code, even if advised of the possibility of such damage. *
'*                                                                                                           *
'* Any views and conclusions contained in the source code and any documentation are those of the author's    *
'* and should not be interpreted as representing official policies,                                          *
'* either expressed or implied, of Crownhill Associates Ltd.                                                 *
'*************************************************************************************************************
'
' Hardware PWM manipulation macros
'
$ifndef _HPWM_INC_
$define _HPWM_INC_

#ifMacro- _HPWM_mSetFreq, _HPWM1_SetFreq, _HPWM2_SetFreq, _HPWM1_mSetDuty, _HPWM2_mSetDuty, _HPWM1_Duty, _HPWM2_Duty
    GoTo _HPWM_Main_                    ' Jump over any subroutines

    Symbol hpwm_cFosc = Xtal * 1000

    Dim hpwm_wFrequency As Word         ' Used to pass the 16-bit frequency value as a parameter
    Dim hpwm_bDuty As Byte              ' Used to pass the 8-bit duty cycle as a paramater
'
' Variables used within the __HPWM_mSetFreq subroutine
'
    Dim hpwm_bPrescaleSelect As Byte
    Dim hpwm_dMaxDuty As Dword
    Dim hpwm_wCalcDuty As Word
    Dim hpwm_wPR2Value As Word
    Dim hpwm_wPRConst As Word
#endIfMacro

'-----------------------------------------------------------------------------
' Start CCP1 operation
' Input     : None
' Output    : None
' Notes     : None
'
$define HPWM1_Start() '
    CCP1CON = $0C     '
    Output __CCP1_PIN

'-----------------------------------------------------------------------------
' Stop CCP1 operation
' Input     : None
' Output    : None
' Notes     : None
'
$define HPWM1_Stop() '
   Input __CCP1_PIN  '
   CCP1CON = $00

'-----------------------------------------------------------------------------
' Helper routine to load the duty cycle into CCPR1L:CCP1CON<5:4>
' Input     : hpwm_wCalcDuty holds the 10-bit value
' Output    : None
' Notes     : CCPR1L contains the eight MSbs, and CCP1CON<5:4> contains the two LSbs.
'           : This 10-Bit value is represented by CCPR1L:CCP1CON<5:4>.
'
$define HPWM1_mSetDuty() _HPWM1_mSetDuty

_HPWM1_mSetDuty Macro-
    GoSub __HPWM1_mSetDuty
    Endm

#ifMacro- _HPWM1_mSetDuty, _HPWM_mSetFreq, _HPWM1_SetFreq, _HPWM1_Duty
__HPWM1_mSetDuty:
    CCP1CON = CCP1CON & %11001111
    WREG = hpwm_wCalcDuty << 4
    WREG = WREG & %00110000
    CCP1CON = CCP1CON | WREG
    CCPR1L = hpwm_wCalcDuty >> 2
    Return
#endIfMacro-
'-----------------------------------------------------------------------------
' Start CCP2 operation
' Input     : None
' Output    : None
' Notes     : None
'
$define HPWM2_Start() '
    CCP2CON = $0C     '
    Output __CCP2_PIN

'-----------------------------------------------------------------------------
' Stop CCP2 operation
' Input     : None
' Output    : None
' Notes     : None
'
$define HPWM2_Stop() '
   Input __CCP2_PIN  '
   CCP2CON = $00

'-----------------------------------------------------------------------------
' Helper routine to load the duty cycle into CCPR2L:CCP2CON<5:4>
' Input     : hpwm_wCalcDuty holds the 10-bit value
' Output    : None
' Notes     : CCPR2L contains the eight MSbs, and CCP2CON<5:4> contains the two LSbs.
'           : This 10-Bit value is represented by CCPR2L:CCP2CON<5:4>.
'
$define HPWM2_mSetDuty() _HPWM2_mSetDuty

_HPWM2_mSetDuty Macro-
    GoSub __HPWM2_mSetDuty
    Endm

#ifMacro- _HPWM2_mSetDuty, _HPWM_mSetFreq, _HPWM2_SetFreq, _HPWM2_Duty
__HPWM2_mSetDuty:
    CCP2CON = CCP2CON & %11001111
    WREG = hpwm_wCalcDuty << 4
    WREG = WREG & %00110000
    CCP2CON = CCP2CON | WREG
    CCPR2L = hpwm_wCalcDuty >> 2
    Return
#endIfMacro-

'-----------------------------------------------------------------------------
$ifdef _CCP3CON
'-----------------------------------------------------------------------------
' Start CCP3 operation
' Input     : None
' Output    : None
' Notes     : None
'
$define HPWM3_Start() '
    CCP3CON = $0C     '
#ifSym __CCP3_PIN     '
    Output __CCP3_PIN '
#endIfSym
'-----------------------------------------------------------------------------
' Stop CCP3 operation
' Input     : None
' Output    : None
' Notes     : None
'
$define HPWM3_Stop() '
#ifSym __CCP3_PIN    '
   Input __CCP4_PIN  '
#endIfSym            '
   CCP3CON = $00
'-----------------------------------------------------------------------------
' Helper routine to load the duty cycle into CCPR3L:CCP3CON<5:4>
' Input     : hpwm_wCalcDuty holds the 10-bit value
' Output    : None
' Notes     : CCPR3L contains the eight MSBs, and CCP3CON<5:4> contains the two LSbs.
'           : This 10-Bit value is represented by CCPR3L:CCP3CON<5:4>.
'
$define HPWM3_mSetDuty() _HPWM3_mSetDuty

_HPWM3_mSetDuty Macro-
    GoSub __HPWM3_mSetDuty
    Endm

#ifMacro- _HPWM3_mSetDuty, _HPWM_mSetFreq, _HPWM3_SetFreq, _HPWM3_Duty
__HPWM3_mSetDuty:
    CCP3CON = CCP3CON & %11001111
    WREG = hpwm_wCalcDuty << 4
    WREG = WREG & %00110000
    CCP3CON = CCP3CON | WREG
    CCPR3L = hpwm_wCalcDuty >> 2
    Return
#endIfMacro-
$endif  ' _CCP3CON

'-----------------------------------------------------------------------------
$ifdef _CCP4CON
'-----------------------------------------------------------------------------
' Start CCP4 operation
' Input     : None
' Output    : None
' Notes     : None
'
$define HPWM4_Start() '
    CCP4CON = $0C     '
#ifSym __CCP4_PIN     '
    Output __CCP4_PIN '
#endIfSym
'-----------------------------------------------------------------------------
' Stop CCP4 operation
' Input     : None
' Output    : None
' Notes     : None
'
$define HPWM4_Stop() '
#ifSym __CCP4_PIN    '
   Input __CCP4_PIN  '
#endIfSym            '
   CCP4CON = $00
'-----------------------------------------------------------------------------
' Helper routine to load the duty cycle into CCPR4L:CCP4CON<5:4>
' Input     : hpwm_wCalcDuty holds the 10-bit value
' Output    : None
' Notes     : CCPR4L contains the eight MSBs, and CCP4CON<5:4> contains the two LSbs.
'           : This 10-Bit value is represented by CCPR4L:CCP4CON<5:4>.
'
$define HPWM4_mSetDuty() _HPWM4_mSetDuty

_HPWM4_mSetDuty Macro-
    GoSub __HPWM4_mSetDuty
    Endm

#ifMacro- _HPWM4_mSetDuty, _HPWM_mSetFreq, _HPWM4_SetFreq, _HPWM4_Duty
__HPWM4_mSetDuty:
    CCP4CON = CCP4CON & %11001111
    WREG = hpwm_wCalcDuty << 4
    WREG = WREG & %00110000
    CCP4CON = CCP4CON | WREG
    CCPR4L = hpwm_wCalcDuty >> 2
    Return
#endIfMacro-
$endif  ' _CCP4CON
'-----------------------------------------------------------------------------
' Helper routine to calculate a duty cycle required for a given PWM frequency
' Input     : hpwm_wFrequency holds the 16-bit frequency of the PWM
'           : hpwm_bPrescaleSelect holds the 8-bit prescale value
'           : hpwm_bDuty holds the 8-bit duty cycle (0 to 255)
' Output    : hpwm_wCalcDuty holds the 16-bit duty cycle value
' Notes     : None
'
#ifMacro- _HPWM_mSetFreq, _HPWM1_SetFreq, _HPWM2_SetFreq, _HPWM1_Duty, _HPWM2_Duty
HPWM_mCalculateDuty:
    hpwm_wCalcDuty = hpwm_wFrequency * hpwm_bPrescaleSelect ' \
    hpwm_wCalcDuty = hpwm_cFosc / hpwm_wCalcDuty            ' / Determine maximum duty
    hpwm_dMaxDuty = hpwm_wCalcDuty * hpwm_bDuty             ' \
    hpwm_wCalcDuty = hpwm_dMaxDuty / 256                    ' / Calculate duty value
    Return
#endIfMacro-
'-----------------------------------------------------------------------------
' Calculate the values required for a given frequency and duty cycle
' Input     : hpwm_wFrequency holds the 16-bit frequency (in Hz)
'           : WREG holds the 8-bit duty cycle (0 to 255)
' Output    : STATUS.0 (Carry flag) holds 1 if the frequency is achievable
' Notes     : None
'
$define HPWM_mSetFreq(pFrequency, pInitialDuty) '
    hpwm_wFrequency = pFrequency                '
    WREG = pInitialDuty                         '
    _HPWM_mSetFreq

_HPWM_mSetFreq Macro-
        GoSub __HPWM_mSetFreq
    Endm

#ifMacro- _HPWM_mSetFreq, _HPWM1_SetFreq, _HPWM2_SetFreq
__HPWM_mSetFreq:
    hpwm_bDuty = WREG
    hpwm_wFrequency = hpwm_wFrequency / 1000
    hpwm_wPRConst = hpwm_cFosc / hpwm_wFrequency
    hpwm_wPRConst = hpwm_wPRConst / 4
'
' Loop through all the valid prescalers
'
    hpwm_bPrescaleSelect = 1
    Repeat
        hpwm_wPR2Value = hpwm_wPRConst / hpwm_bPrescaleSelect           ' \
        hpwm_wPR2Value = hpwm_wPR2Value - 1                             ' / Calculate a PR2 value
        If hpwm_wPR2Value <= 255 Then                                   ' \ Is the value to place into PR2 valid?
            If hpwm_wPR2Value > 1 Then                                  ' /
                T2CON = %00000000                                       ' Yes. So Default to T2CON loaded with 0
                If hpwm_bPrescaleSelect = 4 Then                        ' Is hpwm_bPrescaleSelect 4?
                    T2CON = %00000001                                   ' Yes. So adjust T2CON accordingly
                Else If hpwm_bPrescaleSelect = 16 Then                  ' Is hpwm_bPrescaleSelect 16?
                    T2CON = %00000011                                   ' Yes. So adjust T2CON accordingly
                EndIf
                PR2 = hpwm_wPR2Value                                    ' Initialise PR2
                GoSub HPWM_mCalculateDuty                               ' Calculate the duty value
                Set T2CONbits_TMR2ON                                    ' Start Timer 2
                STATUS.0 = 1                                            ' Return Carry flag holding 1 if successful
                Return                                                  ' Exit the subroutine
            EndIf
        EndIf
        hpwm_bPrescaleSelect = hpwm_bPrescaleSelect * 4                 ' Move to the prescaler select value
    Until hpwm_bPrescaleSelect > 16
    STATUS.0 = 0                                                        ' Return Carry flag holding 0 if unsuccesful
    Return
#endIfMacro-

'-----------------------------------------------------------------------------
' Calculate the values required for a required frequency and duty cycle for CCP1
' Input     : hpwm_wFrequency holds the 16-bit frequency (in Hz)
'           : WREG holds the 8-bit duty cycle (0 to 255)
' Output    : STATUS.0 (Carry flag) holds 1 if the frequency is achievable
' Notes     : Also starts the PWM for CCP1
'
$define HPWM1_SetFreq(pFrequency, pDuty) '
    hpwm_wFrequency = pFrequency         '
    WREG = pDuty                         '
    _HPWM1_SetFreq

_HPWM1_SetFreq Macro-
        GoSub __HPWM1_SetFreq
    Endm

#ifMacro- _HPWM1_SetFreq
__HPWM1_SetFreq:
    HPWM_mSetFreq(hpwm_wFrequency, WREG)        ' Calculate the values for a given frequency and duty cycle
    If STATUS.0 = 1 Then                        ' Was the frequency accepted?
        HPWM1_mSetDuty()                        ' Yes. So load the CCP1 registers with the appropriate value
        HPWM1_Start()                           ' Start PWM1
        STATUS.0 = 1                            ' Indicate everything OK
    EndIf
    Return
#endIfMacro-

'-----------------------------------------------------------------------------
' Calculate the values required for a required frequency and duty cycle for CCP2
' Input     : hpwm_wFrequency holds the 16-bit frequency (in Hz)
'           : WREG holds the 8-bit duty cycle (0 to 255)
' Output    : STATUS.0 (Carry flag) holds 1 if the frequency is achievable
' Notes     : Also starts the PWM For CCP2
'
$define HPWM2_SetFreq(pFrequency, pDuty) '
    hpwm_wFrequency = pFrequency         '
    WREG = pDuty                         '
    _HPWM2_SetFreq

_HPWM2_SetFreq Macro-
        GoSub __HPWM2_SetFreq
    Endm

#ifMacro- _HPWM2_SetFreq
__HPWM2_SetFreq:
    HPWM_mSetFreq(hpwm_wFrequency, WREG)        ' Calculate the values for a given frequency and duty cycle
    If STATUS.0 = 1 Then                        ' Was the frequency accepted?
        HPWM2_mSetDuty()                        ' Yes. So load the CCP2 registers with the appropriate value
        HPWM2_Start()                           ' Start PWM2
        STATUS.0 = 1                            ' Indicate everything OK
    EndIf
    Return
#endIfMacro-

'-----------------------------------------------------------------------------
$ifdef _CCP3CON
'-----------------------------------------------------------------------------
' Calculate the values required for a required frequency and duty cycle for CCP3
' Input     : hpwm_wFrequency holds the 16-bit frequency (in Hz)
'           : WREG holds the 8-bit duty cycle (0 to 255)
' Output    : STATUS.0 (Carry flag) holds 1 if the frequency is achievable
' Notes     : Also starts the PWM For CCP3
'
$define HPWM3_SetFreq(pFrequency, pDuty) '
    hpwm_wFrequency = pFrequency         '
    WREG = pDuty                         '
    _HPWM3_SetFreq

_HPWM3_SetFreq Macro-
        GoSub __HPWM3_SetFreq
    Endm

#ifMacro- _HPWM3_SetFreq
__HPWM3_SetFreq:
    HPWM_mSetFreq(hpwm_wFrequency, WREG)        ' Calculate the values for a given frequency and duty cycle
    If STATUS.0 = 1 Then                        ' Was the frequency accepted?
        HPWM3_mSetDuty()                        ' Yes. So load the CCP3 registers with the appropriate value
        HPWM3_Start()                           ' Start PWM2
        STATUS.0 = 1                            ' Indicate everything OK
    EndIf
    Return
#endIfMacro-
$endif  ' _CCP3CON
'-----------------------------------------------------------------------------
$ifdef _CCP4CON
'-----------------------------------------------------------------------------
' Calculate the values required for a required frequency and duty cycle for CCP4
' Input     : hpwm_wFrequency holds the 16-bit frequency (in Hz)
'           : WREG holds the 8-bit duty cycle (0 to 255)
' Output    : STATUS.0 (Carry flag) holds 1 if the frequency is achievable
' Notes     : Also starts the PWM For CCP4
'
$define HPWM4_SetFreq(pFrequency, pDuty) '
    hpwm_wFrequency = pFrequency         '
    WREG = pDuty                         '
    _HPWM4_SetFreq

_HPWM4_SetFreq Macro-
        GoSub __HPWM4_SetFreq
    Endm

#ifMacro- _HPWM4_SetFreq
__HPWM4_SetFreq:
    HPWM_mSetFreq(hpwm_wFrequency, WREG)        ' Calculate the values for a given frequency and duty cycle
    If STATUS.0 = 1 Then                        ' Was the frequency accepted?
        HPWM4_mSetDuty()                        ' Yes. So load the CCP4 registers with the appropriate value
        HPWM4_Start()                           ' Start PWM2
        STATUS.0 = 1                            ' Indicate everything OK
    EndIf
    Return
#endIfMacro-
$endif  ' _CCP4CON
'-----------------------------------------------------------------------------
' Adjust the duty cycle of CCP1
' Input     : WREG holds the 8-bit duty cycle (0 to 255)
' Output    : None
' Notes     : None
'
$define HPWM1_Duty(pDuty)'
    WREG = pDuty         '
    _HPWM1_Duty

_HPWM1_Duty Macro-
    GoSub __HPWM1_Duty
    Endm

#ifMacro- _HPWM1_Duty
__HPWM1_Duty:
    hpwm_bDuty = WREG
    GoSub HPWM_mCalculateDuty
    HPWM1_mSetDuty()
    Return
#endIfMacro-

'-----------------------------------------------------------------------------
' Adjust the duty cycle of CCP2
' Input     : WREG holds the 8-bit duty cycle (0 to 255)
' Output    : None
' Notes     : None
'
$define HPWM2_Duty(pDuty)'
    WREG = pDuty         '
    _HPWM2_Duty

_HPWM2_Duty Macro-
    GoSub __HPWM2_Duty
    Endm

#ifMacro- _HPWM2_Duty
__HPWM2_Duty:
    hpwm_bDuty = WREG
    GoSub HPWM_mCalculateDuty
    HPWM2_mSetDuty()
    Return
#endIfMacro
'-----------------------------------------------------------------------------
$ifdef _CCP3CON
'-----------------------------------------------------------------------------
' Adjust the duty cycle of CCP3
' Input     : WREG holds the 8-bit duty cycle (0 to 255)
' Output    : None
' Notes     : None
'
$define HPWM3_Duty(pDuty)'
    WREG = pDuty         '
    _HPWM3_Duty

_HPWM3_Duty Macro-
    GoSub __HPWM3_Duty
    Endm

#ifMacro- _HPWM3_Duty
__HPWM3_Duty:
    hpwm_bDuty = WREG
    GoSub HPWM_mCalculateDuty
    HPWM3_mSetDuty()
    Return
#endIfMacro
$endif ' _CCP3CON
'-----------------------------------------------------------------------------
$ifdef _CCP4CON
'-----------------------------------------------------------------------------
' Adjust the duty cycle of CCP4
' Input     : WREG holds the 8-bit duty cycle (0 to 255)
' Output    : None
' Notes     : None
'
$define HPWM4_Duty(pDuty)'
    WREG = pDuty         '
    _HPWM4_Duty

_HPWM4_Duty Macro-
    GoSub __HPWM4_Duty
    Endm

#ifMacro- _HPWM4_Duty
__HPWM4_Duty:
    hpwm_bDuty = WREG
    GoSub HPWM_mCalculateDuty
    HPWM4_mSetDuty()
    Return
#endIfMacro
$endif ' _CCP4CON
'---------------------------------------------------------------------------
_HPWM_Main_:

$endif

